from __future__ import unicode_literals
import os
import re
import torch
from shutil import copytree, ignore_patterns
import onmt.inputters as inputters
from onmt.inputters.dataset_base import Dataset
from onmt.inputters.text_dataset import TextDataReader
from onmt.inputters.inputter import OrderedIterator
from onmt.utils.parse import ArgumentParser
from onmt.model_builder import build_base_model
from onmt.utils.misc import use_gpu
from onmt.utils.optimizers import Optimizer
from onmt.trainer import build_trainer
from onmt.translate import GNMTGlobalScorer
from onmt.models import ModelSaver
from pangeamt_toolkit.pangeanmt.extended_model.extended_model import ExtendedModel
from pangeamt_toolkit.pangeanmt.onmtx_translator import OnmtxTranslator



class Pangeanmt:
    def __init__(self, model_dir):

        # Model dir
        self._model_dir = os.path.abspath(model_dir)
        if not os.path.isdir(self._model_dir):
            msg = f"{model_dir} doesn't exists'"
            raise ValueError(msg)

        # Extended model
        self._extended_model = ExtendedModel(model_dir)

        # Config
        self._config = self._extended_model.config

        # Options
        self._opts = self._config.opts

        # Get the model options
        model_path = self._opts.models[0]
        checkpoint = torch.load(
            model_path,
            map_location=lambda storage, loc: storage)
        self._model_opts = ArgumentParser.ckpt_model_opts(checkpoint['opt'])
        ArgumentParser.update_model_opts(self._model_opts)
        ArgumentParser.validate_model_opts(self._model_opts)

        # Train_steps
        self._train_steps = self._model_opts.train_steps

        # Extract vocabulary
        vocab = checkpoint['vocab']
        if inputters.old_style_vocab(vocab):
            self._fields = inputters.load_old_vocab(
                vocab, self._opts.data_type, dynamic_dict=self._model_opts.copy_attn
            )
        else:
            self._fields = vocab

        # Build model
        self._model = build_base_model(
            self._model_opts,
            self._fields,
            use_gpu(self._opts),
            checkpoint,
            self._opts.gpu)

        if self._opts.fp32:
            self._model.float()

        #Translator
        scorer = GNMTGlobalScorer.from_opt(self._opts)

        self.translator = OnmtxTranslator.from_opt(
            self._model,
            self._fields,
            self._opts,
            self._model_opts,
            global_scorer=scorer,
            out_file=None,
            report_score=False,
            logger=None,
        )

        # Create trainer
        self._optim = Optimizer.from_opt(self._model, self._opts, checkpoint=checkpoint)

        device_id = -1 # TODO Handle GPU
        self.trainer = build_trainer(
            self._opts,
            device_id,
            self._model,
            self._fields,
            self._optim)


    def translate(self, srcs):
        # Set model mode to eval
        self._model.eval()
        self._model.generator.eval()

        # Translate
        translations = self.translator.translate(
            src=srcs,
            batch_size=self._config.opts.batch_size,
        )
        return translations


    def train (self, src, tgt):
        online_learning = self._config.online_learning
        # Check if online learning is active
        if not online_learning.is_active:
            msg = f'Online learning is not active'
            raise ValueError(msg)

        for _ in range(online_learning.steps):
            # Set model mode to train
            self._model.train(True)

            # Create the dataset
            src_reader = TextDataReader()
            tgt_reader = TextDataReader()

            srcs = [src]
            tgts = [tgt]

            dataset = Dataset(
                self._fields,
                readers=[src_reader, tgt_reader],
                data=[("src", srcs), ("tgt", tgts)],
                dirs=[None, None],
                sort_key=None,
                filter_pred=None
            )

            # Create batches
            batches = OrderedIterator(dataset, batch_size=1)

            # Update train steps
            self._train_steps += 1

            # Train the model.
            self.trainer.train(
                batches,
                self._train_steps,
                save_checkpoint_steps=None,
                valid_iter=None,
                valid_steps=None)

    def save_model(self, new_model_dir):
        new_model_dir = os.path.abspath(new_model_dir)
        if os.path.isdir(new_model_dir):
            if not os.listdir:
                os.rmdir(new_model_dir)
            else:
                msg = f"{new_model_dir} already exists"
                raise ValueError(msg)
        copytree(self._extended_model.model_dir, new_model_dir, ignore=ignore_patterns('*.pt'))
        model_filename = os.path.basename(self._extended_model.config.opts.models[0])
        model_filename_base = re.sub('_[0-9]*\.pt$', '', model_filename )
        base_path = os.path.join(new_model_dir, model_filename_base)
        model_saver = ModelSaver(
            base_path,
            self._model,
            self._model_opts,
            self._fields,
            self._optim,
            keep_checkpoint=-1
        )
        model_saver.save(self._train_steps)



from sacremoses import MosesTokenizer as _MosesTokenizer
from sacremoses import MosesDetokenizer as _MosesDetokenizer
from MeCab import Tagger as _Tagger

class MecabTokenizerProcessor:

    def __init__(self):
        self._mod = 'tok'
        self._tok = _Tagger('-Owakati')

    def get_mod(self):
        return self._mod
    mod = property(get_mod)

    def tokenize(self, str, escape=None):
        """ Applies mecab tokenization to str
        """
        t = self._tok.parse(str).split(' ')
        #print(t, '\n', t[:-1])
        return t[:-1]

    def detokenize(self, str_array):
        """ Joins str_array
        """
        return ('').join(str_array)

    def preprocess(self, seg):
        seg.src = self.tokenize(seg.src)

    def preprocess_str(self, str):
        return self.tokenize(str)

    def postprocess(self, seg):
        seg.tgt = self.detokenize(seg.tgt)

    def postprocess_str(self, str):
        return self.detokenize(str)



import time
from aiohttp import web as _web
from pangeamt_toolkit import Seg

async def translate(req):
    nmt = req.app['nmt']
    pipeline = req.app['pipeline']
    lock = req.app['lock']
    sem = req.app['sem']
    log = req.app['log_path']

    try:
        req = await req.json()
        batch_to_trans = []
        segs = []
        async with sem:
            for tu in req.get('tus'):
                seg = Seg(tu['src'])
                pipeline.preprocess(seg)
                batch_to_trans.append(seg.src)
                segs.append(seg)

        async with lock:
            translations = nmt.translate(batch_to_trans)

        for tu in req.get('tus'):
            named_tuple = time.localtime() # get struct_time
            time_string = time.strftime("%m/%d/%Y, %H:%M:%S", named_tuple)
            seg = segs.pop(0)
            translation = translations.pop(0)
            seg.tgt = seg.tgt_raw = (' ').join(translation.tgt)
            pipeline.postprocess(seg)
            with open(log, 'a+') as file:
                words = ['source:', 'source_prep:', 'translation:',
                    'translation_post:', 'score:']
                file.write(f'----Translating----\n'\
                    f'{time_string}\n'\
                    f'{words[0]:>17} {seg.src_raw}\n'\
                    f'{words[1]:>17} {seg.src}\n'\
                    f'{words[2]:>17} {seg.tgt_raw}\n'\
                    f'{words[3]:>17} {seg.tgt}\n'\
                    f'{words[4]:>17} {translation.pred_score}\n\n')
            tu['tgt'] = seg.tgt

        return _web.json_response(req, status=200)

    except Exception as e:
        response_obj = {'status': 'failed', 'reason': str(e)}
        with open(log, 'a+') as file:
            named_tuple = time.localtime()
            time_string = time.strftime("%m/%d/%Y, %H:%M:%S", named_tuple)
            file.write(f'--Failed Translation--\n'\
                f'{time_string}\n'\
                f'reason: {str(e)}\n\n')
        return _web.json_response(response_obj, status=500)



import neologdn as _neologdn

class JapaneseNormalizerProcessor:

    def __init__(self):
        self._mod = 'janorm'

    def get_mod(self):
        return self._mod
    mod = property(get_mod)

    def preprocess(self, seg):
        seg.src = _neologdn.normalize(seg.src, repeat=1)

    def preprocess_str(self, str):
        return _neologdn.normalize(str, repeat=1)

    def postprocess(self, seg):
        seg.tgt = seg.tgt

    def postprocess_str(self, str):
        return str



from __future__ import unicode_literals
import json
import os
from onmt.opts import train_opts as _train_opts, translate_opts as _translate_opts, model_opts as _model_opts
from onmt.utils.parse import ArgumentParser



class Config:
    SRC_LANG = 'src_lang'
    TGT_LANG = 'tgt_lang'
    ONLINE_LEARNING = 'online_learning'

    def __init__(self, model_dir):

        # Model dir
        self._model_dir = model_dir

        # Get pt file
        model_files = []
        for file in os.listdir(self._model_dir):
            if file.endswith(".pt"):
                model_files.append(os.path.join(model_dir,file))
        if len(model_files) != 1:
            msg = f"Extended model {self._model_dir} sould have one .pt file. {len(model_files)}"
            raise ValueError(msg)
        model_file = model_files[0]

        # Load config from file
        config_path = os.path.join(model_dir, "config.json")

        # Load config from json file
        with open(config_path) as f:
            config = json.load(f)

        # Langs
        self._src_lang = config[self.SRC_LANG]
        self._tgt_lang = config[self.TGT_LANG]

        # Online learning
        if  self.ONLINE_LEARNING in config:
            self._online_learning = _OnlineLearningConfig(config[self.ONLINE_LEARNING])
        else:
            self._online_learning = _OnlineLearningConfig()

        # Create a parser for train and translate options
        parser = ArgumentParser(description='pangeanmt options',  conflict_handler='resolve')

        # Parser for translate options
        _translate_opts(parser)

        # Parser for training options
        _train_opts(parser)

        # Parser for model options
        _model_opts(parser)

        # --src argument is not used
        parser.add_argument('--src', '-src', required=False,
                            help="This argument isn't used!")

        # --data argument is not used
        parser.add_argument('--data', '-data', required=False,
                            help="This argument isn't used!")

        # --seed Overwrite default
        parser.add_argument('--seed', '-seed', required=False, default=829,
                            help="Seed")

        # Create opts from config
        args = ['--model', model_file]
        for k, v in config['opts'].items():
            args.append('--'+ k)
            if v is not None:
                if k == 'model':
                    v = os.path.join(model_dir, v)
                args.append(str(v))
        self._opts = parser.parse_args(args)

    def get_src_lang(self):
        return self._src_lang
    src_lang = property(get_src_lang)

    def get_tgt_lang(self):
        return self._tgt_lang
    tgt_lang = property(get_tgt_lang)

    def get_opts(self):
        return self._opts
    opts = property(get_opts)

    def get_online_learning(self):
        return self._online_learning
    online_learning = property(get_online_learning)


class _OnlineLearningConfig:
    ACTIVE = 'active'
    STEPS = 'steps'
    def __init__(self, opts=None):
        if opts is None:
            self._active = False
            self._steps = 0
        else:
            self._active = opts[self.ACTIVE]
            self._steps = opts[self.STEPS]

    def get_active(self):
        return self._active
    is_active = property(get_active)

    def get_steps(self):
        return self._steps
    steps = property(get_steps)



import os
import yaml as _yaml
from re import sub as _sub

class Pipeline:

    def __init__(self, processors):
        """ processors is a list of dicts from the general config file
            with this structure:
                {
                    'name': processor's shortname,
                    'args': [list of the processor's args]
                }
        """
        self._processes = []
        for process in processors:
            processor = self._PROCESSORS[process['name']]
            klass_name = processor['class']
            path = processor['path']
            args = process['args']
            mod = __import__(path, fromlist=[klass_name])
            klass = getattr(mod, klass_name)
            processor = klass(*args)
            self._processes.append(processor)

    def preprocess(self, seg):
        for process in self._processes:
            process.preprocess(seg)

    def preprocess_str(self, str):
        for process in self._processes:
            try:
                str, _ = process.preprocess_str(str)
            except:
                str = process.preprocess_str(str)
        return str

    def preprocess_file(self, src_path, tgt_path=None):
        if tgt_path:
            with open(src_path, 'r') as src_file:
                with open(tgt_path, 'w+') as tgt_file:
                    for line in src_file:
                        line = self.preprocess_str(line)
                        if line[-1:] == '\n':
                            tgt_file.write(line)
                        else:
                            tgt_file.write(line + '\n')
        else:
            src_path_list = src_path.split('/')
            src_mods = src_path_list[-1].split('.')[1:-1]
            src_name_raw = src_path_list[-1].split('.')[0]
            src_lang = src_path_list[-1].split('.')[-1]
            path = ('/').join(src_path_list[:-1])
            for process in self._processes:
                if process.mod not in src_mods:
                    src_mods.append(process.mod)
                    tgt_path = f"{path}/{src_name_raw}.{('.').join(src_mods)}."\
                        f'{src_lang}'
                    self._preprocess_file(process, src_path, tgt_path)
                    src_path = tgt_path
                else:
                    pass


    def _preprocess_file(self, process, src_path, tgt_path):
        if os.path.exists(tgt_path):
            pass
        else:
            with open(src_path, 'r') as src_file:
                with open(tgt_path, 'w+') as tgt_file:
                    for line in src_file:
                        try:
                            line, _ = process.preprocess_str(line)
                        except:
                            line = process.preprocess_str(line)
                        if line[-1:] == '\n':
                            tgt_file.write(line)
                        else:
                            tgt_file.write(line + '\n')

    def postprocess(self, seg):
        processes = self._processes[::-1]
        for process in processes:
            process.postprocess(seg)

    def postprocess_str(self, str):
        processes = self._processes[::-1]
        for process in processes:
            str = process.postprocess_str(str)
        return str

    def postprocess_file(self, src_path, tgt_path):
        with open(src_path, 'r') as src_file:
            with open(tgt_path, 'w+') as tgt_file:
                for line in src_file:
                    line = self.postprocess_str(line)
                    tgt_file.write(line + '\n')

    _PROCESSORS = {
        'tokenize':{
            'class': 'GeneralTokenizerProcessor',
            'path': 'pangeamt_toolkit.processors.general_tokenizer_processor'
        },

        'moses_tokenize':{
            'class': 'MosesTokenizerProcessor',
            'path': 'pangeamt_toolkit.processors.moses_tokenizer_processor'
        },

        'jieba_tokenize':{
            'class': 'JiebaTokenizerProcessor',
            'path': 'pangeamt_toolkit.processors.jieba_tokenizer_processor'
        },

        'mecab_tokenize':{
            'class': 'MecabTokenizerProcessor',
            'path': 'pangeamt_toolkit.processors.mecab_tokenizer_processor'
        },

        'truecase':{
            'class': 'MosesTruecasingProcessor',
            'path': 'pangeamt_toolkit.processors.moses_truecasing_processor'
        },

        'normalizer':{
            'class': 'MosesNormalizerProcessor',
            'path': 'pangeamt_toolkit.processors.moses_normalizer_processor'
        },

        'bpe':{
            'class': 'BPEProcessor',
            'path': 'pangeamt_toolkit.processors.bpe_processor'
        },

        'placeholder':{
            'class': 'PlaceholderProcessor',
            'path': 'pangeamt_toolkit.processors.placeholder_processor'
        },

        'punct':{
            'class': 'PunctProcessor',
            'path': 'pangeamt_toolkit.processors.punct_processor'
        },

        'japanese_normalizer':{
            'class': 'JapaneseNormalizerProcessor',
            'path': 'pangeamt_toolkit.processors.japanese_normalizer_processor'
        },
    }



from __future__ import print_function
from onmt.translate.translator import Translator as _Translator
import onmt.model_builder
import onmt.translate.beam
from onmt.translate import TranslationBuilder as _TransBuilder
import onmt.inputters as inputters
import onmt.decoders.ensemble
from pangeamt_toolkit.pangeanmt.onmtx_translation import OnmtxTranslation

class OnmtxTranslator(_Translator):

    def translate(
            self,
            src,
            with_attn=True,
            batch_size=None,
            phrase_table=""):

        src_dir=None

        attn_debug = False
        if with_attn:
            attn_debug = True

        if batch_size is None:
            raise ValueError("batch_size must be set")

        data = inputters.Dataset(
            self.fields,
            readers=([self.src_reader]),
            data=[("src", src)],
            dirs=[src_dir],
            sort_key=inputters.str2sortkey[self.data_type],
            filter_pred=self._filter_pred
        )

        data_iter = inputters.OrderedIterator(
            dataset=data,
            device=self._dev,
            batch_size=batch_size,
            train=False,
            sort=False,
            sort_within_batch=False,
            shuffle=False
        )

        # Translate
        xlation_builder = _TransBuilder(
            data, self.fields, self.n_best, self.replace_unk, None,
            self.phrase_table
        )

        results = []
        for batch in data_iter:

            batch_data = self.translate_batch(
                batch, data.src_vocabs, attn_debug
            )
            translations = xlation_builder.from_batch(batch_data)
            for translation in translations:
                simple_translation = OnmtxTranslation(
                    translation.src,
                    translation.src_raw,
                    translation.pred_sents[0],
                    translation.attns[0],
                    translation.pred_scores[0]
                )
                results.append(simple_translation)
        return results



class Seg:
    def __init__(self, src):
        self._src = src
        self._src_raw = src
        self._tgt = None
        self._tgt_raw = None
        self._error = None
        self._src_entities = []
        self._tgt_entities = []
        self._src_case = None
        self._pred_score = 0
        self._attention = None

    def get_src_entities(self):
        return self._src_entities

    def set_src_entities(self, src_entities):
        self._src_entities = src_entities

    src_entities = property(get_src_entities, set_src_entities)


    def get_tgt_entities(self):
        return self._tgt_entities

    def set_tgt_entities(self, tgt_entities):
        self._tgt_entities = tgt_entities

    tgt_entities = property(get_tgt_entities, set_tgt_entities)


    # def preprocess(self, detector):
    #     preprocessed = self._src
    #     if detector is not None:
    #         self._detection = detector.detect(self._src)
    #         preprocessed = self._detection.get_text_with_place_holders()
    #     return preprocessed
    #
    # def postprocess(self):
    #     postprocessed = self.tgt_raw
    #     if self._detection is not None:
    #         # Fist we repace all place holder uuid with associated unicode private char
    #         for replace, match in self._detection.matches.items():
    #             postprocessed = postprocessed.replace(match.id, replace)
    #
    #         # We remove opennmt repetitions
    #         postprocessed = remove_open_nmt_repetition(postprocessed)
    #
    #         # We replace place holder by  appropriate value
    #         for _, match in self._detection.matches.items():
    #             if match.type == Match.TYPE_CHINESE_NUMBER:
    #                 # Hack remove "-" in 4 -brothers
    #                 regex = re.compile(match.replace +  r'(( +)(-)([^0-9]))*')
    #                 replace =  match.translate(self.request.tgt_lang) + r'\2\4'
    #                 postprocessed = re.sub(regex,replace, postprocessed)
    #             elif match.type == Match.TYPE_NUMBER:
    #                 postprocessed= postprocessed.replace(match.replace, match.text)
    #     return postprocessed



    def get_src(self):
        return self._src

    def set_src(self, src):
        self._src = src

    src = property(get_src, set_src)

    def get_src_raw(self):
        return self._src_raw

    src_raw = property(get_src_raw)

    def get_tgt(self):
        return self._tgt

    def set_tgt(self, tgt):
        self._tgt = tgt

    tgt = property(get_tgt, set_tgt)


    def get_tgt_raw(self):
        return self._tgt_raw

    def set_tgt_raw(self, tgt_raw):
        self._tgt_raw = tgt_raw

    tgt_raw = property(get_tgt_raw, set_tgt_raw)


    def get_error(self):
        return self._error

    def set_error(self, error):
        self._error = error
    error = property(get_error, set_error)


    def get_src_case(self):
        return self._src_case

    def set_src_case(self, case):
        self._src_case = case
    src_case = property(get_src_case, set_src_case)


    def get_pred_score(self):
        return self._pred_score

    def set_pred_score(self, pred_score):
        self._pred_score = pred_score
    pred_score = property(get_pred_score, set_pred_score)


    def get_attention(self):
        return self._attention

    def set_attention(self, attention):
        self._attention = attention
    attention = property(get_attention, set_attention)



from pangeamt_toolkit.pangeanmt.request_handler.save import save
from pangeamt_toolkit.pangeanmt.request_handler.train import train
from pangeamt_toolkit.pangeanmt.request_handler.ready import ready
from pangeamt_toolkit.pangeanmt.request_handler.translate import translate

__all__ = ['save', 'ready', 'train', 'translate']



from sacremoses import MosesTokenizer as _MosesTokenizer
from sacremoses import MosesDetokenizer as _MosesDetokenizer
from pangeamt_toolkit.processors import MecabTokenizerProcessor\
    as _Mecab
from pangeamt_toolkit.processors import JiebaTokenizerProcessor\
    as _Jieba

class GeneralTokenizerProcessor:

    def __init__(self, src_lang, tgt_lang=None):
        self._mod = 'tok'
        if src_lang == 'ja':
            self._mtk = _Mecab()
        elif src_lang in ['zh', 'tw']:
            self._mtk = _Jieba()
        else:
            self._mtk = _MosesTokenizer(lang=src_lang)

        if tgt_lang == 'ja':
            self._mdk = _Mecab()
        elif tgt_lang == 'ch':
            self._mdk = _Jieba()
        elif tgt_lang:
            self._mdk = _MosesDetokenizer(lang=tgt_lang)
        else:
            self._mdk = None

    def get_mod(self):
        return self._mod
    mod = property(get_mod)

    def preprocess(self, seg):
        """ Applies the MosesTokenizer to seg.src
        """
        seg.src = (' ').join(self._mtk.tokenize(seg.src, escape=False))

    def preprocess_str(self, str):
        return (' ').join(self._mtk.tokenize(str, escape=False))

    def postprocess(self, seg):
        """ Applies the MosesDetokenizer to seg.tgt
        """
        seg.tgt = self._mdk.detokenize(seg.tgt.split(' '))

    def postprocess_str(self, str):
        return self._mdk.detokenize(str.split(' '))



from distutils.core import setup
setup(
    name = 'pangeamt_toolkit',
    packages = ['pangeamt_toolkit', 'pangeamt_toolkit.pangeanmt',
        'pangeamt_toolkit.pangeanmt.extended_model',
        'pangeamt_toolkit.pangeanmt.request_handler', 'pangeamt_toolkit.seg',
        'pangeamt_toolkit.processors'],
    scripts = ['bin/pangeamt_file_postprocess.py', 'bin/pangeamt_train.py',
    'bin/pangeamt_file_preprocess.py', 'bin/pangeamt_preprocess.py',
    'bin/pangeamt_translate.py', 'bin/pangeamt_multi_bleu.perl',
    'bin/pangeamt_build_server.py', 'bin/pangeamt_files_preprocess.py',
    'bin/pangeamt_norm_tok_preprocess.py', 'bin/pangeamt_truecase_preprocess.py'],
    version = '3.1',
    license='MIT',
    description = 'PANGEAMT-TOOLKIT',
    author = 'PANGEAMT',
    author_email = 'a.cerda@pangeanic.es',
    url = 'https://github.com/Pangeamt/pangeamt_toolkit',
    download_url = 'https://github.com/Pangeamt/pangeamt_toolkit/archive/3.1.tar.gz',
    install_requires=[
        'aiohttp==3.5.4',
        'async-timeout==3.0.1',
        'attrs==19.1.0',
        'certifi==2019.6.16',
        'chardet==3.0.4',
        'Click==7.0',
        'ConfigArgParse==0.14.0',
        'future==0.17.1',
        'idna==2.8',
        'idna-ssl==1.1.0',
        'jieba==0.39',
        'joblib==0.13.2',
        'mecab-python3==0.996.2',
        'multidict==4.5.2',
        'neologdn==0.4',
        'numpy==1.17.0',
        'OpenNMT-py==0.9.1.1',
        'PyYAML==5.1.1',
        'requests==2.22.0',
        'sacremoses==0.0.33', # 0.0.22
        'six==1.12.0',
        'subword-nmt==0.3.6',
        'torch==1.1.0',
        'torchtext==0.4.0',
        'tqdm==4.32.2',
        'typing-extensions==3.7.4',
        'urllib3==1.25.3',
        'uvloop==0.12.2',
        'yarl==1.3.0'
    ]
)



from aiohttp import web

async def ready(req):

    try:
        resp = {
            'rc': 0
        }
        return web.json_response(resp, status=200)

    except Exception as e:
        response_obj = {'status': 'failed', 'reason': str(e)}
        return web.json_response(response_obj, status=500)



import os
import sys
import json
import time
import uvloop
import asyncio
from aiohttp import web
from pangeamt_toolkit.pangeanmt.request_handler.save import save
from pangeamt_toolkit.pangeanmt.request_handler.train import train
from pangeamt_toolkit.pangeanmt.request_handler.ready import ready
from pangeamt_toolkit.pangeanmt.request_handler.translate import translate
from pangeamt_toolkit import Pipeline
from pangeamt_toolkit.pangeanmt import Pangeanmt

asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

current_dir = os.path.abspath(os.path.dirname(__file__))
os.chdir(current_dir)

class PangeanmtServer:
    def __init__(self, engine_path):
        self._app = web.Application()
        model_path = engine_path + "/extended_model"
        config_path = model_path + '/config.json'
        log_path = self._set_up_log(engine_path)

        with open(log_path, 'a+') as file:
            named_tuple = time.localtime()
            time_string = time.strftime("%m/%d/%Y, %H:%M:%S", named_tuple)
            file.write(f'--Starting Server--\n'\
                f'{time_string}\n\n')

        with open(config_path, 'r') as file:
            config = json.loads(file.read())

        self._app['nmt'] = Pangeanmt(model_path) # config['model'] ?
        self._app['pipeline'] = Pipeline(config['pipeline_config'])
        self._app['pipeline_tgt'] = Pipeline(config['pipeline_config_tgt'])
        self._app['engine_path'] = engine_path
        self._app['lock'] = asyncio.Lock()
        self._app['sem'] = asyncio.Semaphore()
        self._app['ol'] = config['online_learning']['active']
        self._app['log_path'] = log_path

        self._app.router.add_post('/save', save)
        self._app.router.add_post('/train', train)
        self._app.router.add_post('/isready', ready)
        self._app.router.add_post('/translate', translate)

    def start(self):
        web.run_app(self._app, port=8081)

    def _set_up_log(self, engine_path):
        while engine_path[-1] == '/':
            engine_path = engine_path[:-1]
        model_name = engine_path.split('/')[-1]
        log_name = f'log_{model_name}.txt'
        logs_dir = f'{("/").join(engine_path.split("/")[:-1])}/logs/'
        print(logs_dir)
        try:
            os.mkdir(logs_dir)
        except:
            pass
        return f'{logs_dir}/{log_name}'
